@@ -19,15 +19,15 @@
  */
 
 #include <config.h>
-#include "exif-mnote-data.h"
 
-#include "exif-data.h"
-#include "exif-ifd.h"
-#include "exif-mnote-data-priv.h"
-#include "exif-utils.h"
-#include "exif-loader.h"
-#include "jpeg-marker.h"
+#include <libexif/exif-mnote-data.h>
+#include <libexif/exif-data.h>
+#include <libexif/exif-ifd.h>
+#include <libexif/exif-mnote-data-priv.h>
+#include <libexif/exif-utils.h>
+#include <libexif/exif-loader.h>
 #include <libexif/exif-log.h>
+#include <libexif/i18n.h>
 
 #include <libexif/olympus/exif-mnote-data-olympus.h>
 #include <libexif/canon/exif-mnote-data-canon.h>
@@ -37,13 +37,17 @@
 #include <stdio.h>
 #include <string.h>
 
-#undef MAX
-#define MAX(a, b)  (((a) > (b)) ? (a) : (b))
-
-#ifdef __WATCOMC__
+#if defined(__WATCOMC__) || defined(_MSC_VER)
 #      define strncasecmp strnicmp
 #endif
 
+#undef JPEG_MARKER_SOI
+#define JPEG_MARKER_SOI  0xd8
+#undef JPEG_MARKER_APP0
+#define JPEG_MARKER_APP0 0xe0
+#undef JPEG_MARKER_APP1
+#define JPEG_MARKER_APP1 0xe1
+
 static const unsigned char ExifHeader[] = {0x45, 0x78, 0x69, 0x66, 0x00, 0x00};
 
 struct _ExifDataPrivate
@@ -59,6 +63,9 @@ struct _ExifDataPrivate
 
 	/* Temporarily used while loading data */
 	unsigned int offset_mnote;
+
+	ExifDataOption options;
+	ExifDataType data_type;
 };
 
 static void *
@@ -66,10 +73,12 @@ exif_data_alloc (ExifData *data, unsigned int i)
 {
 	void *d;
 
-	if (!data || !i) return NULL;
+	if (!data || !i) 
+		return NULL;
 
 	d = exif_mem_alloc (data->priv->mem, i);
-	if (d) return d;
+	if (d) 
+		return d;
 
 	EXIF_LOG_NO_MEMORY (data->priv->log, "ExifData", i);
 	return NULL;
@@ -98,12 +107,17 @@ exif_data_new_mem (ExifMem *mem)
 	ExifData *data;
 	unsigned int i;
 
-	if (!mem) return NULL;
+	if (!mem) 
+		return NULL;
 
 	data = exif_mem_alloc (mem, sizeof (ExifData));
-	if (!data) return (NULL);
+	if (!data) 
+		return (NULL);
 	data->priv = exif_mem_alloc (mem, sizeof (ExifDataPrivate));
-	if (!data->priv) { exif_mem_free (mem, data); return (NULL); }
+	if (!data->priv) { 
+	  	exif_mem_free (mem, data); 
+		return (NULL); 
+	}
 	data->priv->ref_count = 1;
 
 	data->priv->mem = mem;
@@ -118,6 +132,13 @@ exif_data_new_mem (ExifMem *mem)
 		data->ifd[i]->parent = data;
 	}
 
+	/* Default options */
+	exif_data_set_option (data, EXIF_DATA_OPTION_IGNORE_UNKNOWN_TAGS);
+	exif_data_set_option (data, EXIF_DATA_OPTION_FOLLOW_SPECIFICATION);
+
+	/* Default data type: none */
+	exif_data_set_data_type (data, EXIF_DATA_TYPE_COUNT);
+
 	return (data);
 }
 
@@ -131,7 +152,7 @@ exif_data_new_from_data (const unsigned char *data, unsigned int size)
 	return (edata);
 }
 
-static void
+static int
 exif_data_load_data_entry (ExifData *data, ExifEntry *entry,
 			   const unsigned char *d,
 			   unsigned int size, unsigned int offset)
@@ -152,15 +173,17 @@ exif_data_load_data_entry (ExifData *data, ExifEntry *entry,
 	 */
 	s = exif_format_get_size (entry->format) * entry->components;
 	if (!s)
-		return;
+		return 0;
 	if (s > 4)
 		doff = exif_get_long (d + offset + 8, data->priv->order);
 	else
 		doff = offset + 8;
 
-	/* Sanity check */
+	/* Sanity checks */
+	if ((doff + s < doff) || (doff + s < s))
+		return 0;
 	if (size < doff + s)
-		return;
+		return 0;
 
 	entry->data = exif_data_alloc (data, s);
 	if (entry->data) {
@@ -168,21 +191,18 @@ exif_data_load_data_entry (ExifData *data, ExifEntry *entry,
 		memcpy (entry->data, d + doff, s);
 	}
 
-#ifndef EXIF_DONT_CHANGE_MAKER_NOTE
 	/* If this is the MakerNote, remember the offset */
 	if (entry->tag == EXIF_TAG_MAKER_NOTE) {
 		if (entry->size > 6) exif_log (data->priv->log,
-			  EXIF_LOG_CODE_DEBUG, "ExifData",
-		          "MakerNote found (%02x %02x %02x %02x "
-			  "%02x %02x %02x...).",
-			  entry->data[0], entry->data[1], entry->data[2],
-			  entry->data[3], entry->data[4], entry->data[5],
-			  entry->data[6]);
+					       EXIF_LOG_CODE_DEBUG, "ExifData",
+					       "MakerNote found (%02x %02x %02x %02x "
+					       "%02x %02x %02x...).",
+					       entry->data[0], entry->data[1], entry->data[2],
+					       entry->data[3], entry->data[4], entry->data[5],
+					       entry->data[6]);
 		data->priv->offset_mnote = doff;
 	}
-#endif
-
-	exif_entry_fix (entry);
+	return 1;
 }
 
 static void
@@ -192,7 +212,8 @@ exif_data_save_data_entry (ExifData *data, ExifEntry *e,
 {
 	unsigned int doff, s;
 
-	if (!data || !data->priv) return;
+	if (!data || !data->priv) 
+		return;
 
 	/*
 	 * Each entry is 12 bytes long. The memory for the entry has
@@ -203,14 +224,16 @@ exif_data_save_data_entry (ExifData *data, ExifEntry *e,
 	exif_set_short (*d + 6 + offset + 2,
 			data->priv->order, (ExifShort) e->format);
 
-	/* If this is the maker note tag, update it. */
-	if ((e->tag == EXIF_TAG_MAKER_NOTE) && data->priv->md) {
-		exif_mem_free (data->priv->mem, e->data);
-		e->data = NULL;
-		e->size = 0;
-		exif_mnote_data_set_offset (data->priv->md, *ds - 6);
-		exif_mnote_data_save (data->priv->md, &e->data, &e->size);
-		e->components = e->size;
+	if (!(data->priv->options & EXIF_DATA_OPTION_DONT_CHANGE_MAKER_NOTE)) {
+		/* If this is the maker note tag, update it. */
+		if ((e->tag == EXIF_TAG_MAKER_NOTE) && data->priv->md) {
+			exif_mem_free (data->priv->mem, e->data);
+			e->data = NULL;
+			e->size = 0;
+			exif_mnote_data_set_offset (data->priv->md, *ds - 6);
+			exif_mnote_data_save (data->priv->md, &e->data, &e->size);
+			e->components = e->size;
+		}
 	}
 
 	exif_set_long  (*d + 6 + offset + 4,
@@ -224,19 +247,30 @@ exif_data_save_data_entry (ExifData *data, ExifEntry *e,
 	if (s > 4) {
 		doff = *ds - 6;
 		*ds += s;
+
+		/*
+		 * According to the TIFF specification,
+		 * the offset must be an even number. If we need to introduce
+		 * a padding byte, we set it to 0.
+		 */
+		if (s & 1) 
+			(*ds)++;
 		*d = exif_mem_realloc (data->priv->mem, *d, *ds);
 		if (!*d) {
 			EXIF_LOG_NO_MEMORY (data->priv->log, "ExifData", *ds);
 		  	return;
 		}
-		exif_set_long (*d + 6 + offset + 8,
-			       data->priv->order, doff);
+		exif_set_long (*d + 6 + offset + 8, data->priv->order, doff);
+		if (s & 1) 
+			*(*d + *ds - 1) = '\0';
+
 	} else
 		doff = offset + 8;
 
 	/* Write the data. Fill unneeded bytes with 0. */
 	memcpy (*d + 6 + doff, e->data, s);
-	if (s < 4) memset (*d + 6 + doff + s, 0, (4 - s));
+	if (s < 4) 
+		memset (*d + 6 + doff + s, 0, (4 - s));
 }
 
 static void
@@ -249,17 +283,37 @@ exif_data_load_data_thumbnail (ExifData *data, const unsigned char *d,
 			  (int) ds, (int) offset, (int) size);
 		return;
 	}
-	if (data->data) exif_mem_free (data->priv->mem, data->data);
+	if (data->data) 
+		exif_mem_free (data->priv->mem, data->data);
 	data->size = size;
 	data->data = exif_data_alloc (data, data->size);
-	if (!data->data) return;
+	if (!data->data) 
+		return;
 	memcpy (data->data, d + offset, data->size);
 }
 
+#undef CHECK_REC
+#define CHECK_REC(i) 					\
+if ((i) == ifd) {				\
+	exif_log (data->priv->log, EXIF_LOG_CODE_DEBUG, \
+		"ExifData", "Recursive entry in IFD "	\
+		"'%s' detected. Skipping...",		\
+		exif_ifd_get_name (i));			\
+	break;						\
+}							\
+if (data->ifd[(i)]->count) {				\
+	exif_log (data->priv->log, EXIF_LOG_CODE_DEBUG,	\
+		"ExifData", "Attemt to load IFD "	\
+		"'%s' multiple times detected. "	\
+		"Skipping...",				\
+		exif_ifd_get_name (i));			\
+	break;						\
+}
+
 static void
-exif_data_load_data_content (ExifData *data, ExifContent *ifd,
+exif_data_load_data_content (ExifData *data, ExifIfd ifd,
 			     const unsigned char *d,
-			     unsigned int ds, unsigned int offset)
+			     unsigned int ds, unsigned int offset, unsigned int recursion_depth)
 {
 	ExifLong o, thumbnail_offset = 0, thumbnail_length = 0;
 	ExifShort n;
@@ -267,17 +321,28 @@ exif_data_load_data_content (ExifData *data, ExifContent *ifd,
 	unsigned int i;
 	ExifTag tag;
 
-	if (!data || !data->priv) return;
+	if (!data || !data->priv) 
+		return;
+	if ((ifd < 0) || (ifd >= EXIF_IFD_COUNT)) 
+		return;
+
+	if (recursion_depth > 150) {
+		exif_log (data->priv->log, EXIF_LOG_CODE_CORRUPT_DATA, "ExifData",
+			  "Deep recursion detected!");
+		return;
+	}
 
 	/* Read the number of entries */
-	if (offset >= ds - 1) return;
+	if (offset >= ds - 1) 
+		return;
 	n = exif_get_short (d + offset, data->priv->order);
 	exif_log (data->priv->log, EXIF_LOG_CODE_DEBUG, "ExifData",
 	          "Loading %i entries...", n);
 	offset += 2;
 
 	/* Check if we have enough data. */
-	if (offset + 12 * n > ds) n = (ds - offset) / 12;
+	if (offset + 12 * n > ds) 
+		n = (ds - offset) / 12;
 
 	for (i = 0; i < n; i++) {
 
@@ -292,30 +357,30 @@ exif_data_load_data_content (ExifData *data, ExifContent *ifd,
 					   data->priv->order);
 			switch (tag) {
 			case EXIF_TAG_EXIF_IFD_POINTER:
-				exif_data_load_data_content (data,
-					data->ifd[EXIF_IFD_EXIF], d, ds, o);
+				CHECK_REC (EXIF_IFD_EXIF);
+				exif_data_load_data_content (data, EXIF_IFD_EXIF, d, ds, o, recursion_depth + 1);
 				break;
 			case EXIF_TAG_GPS_INFO_IFD_POINTER:
-				exif_data_load_data_content (data,
-					data->ifd[EXIF_IFD_GPS], d, ds, o);
+				CHECK_REC (EXIF_IFD_GPS);
+				exif_data_load_data_content (data, EXIF_IFD_GPS, d, ds, o, recursion_depth + 1);
 				break;
 			case EXIF_TAG_INTEROPERABILITY_IFD_POINTER:
-				exif_data_load_data_content (data,
-					data->ifd[EXIF_IFD_INTEROPERABILITY], d, ds, o);
+				CHECK_REC (EXIF_IFD_INTEROPERABILITY);
+				exif_data_load_data_content (data, EXIF_IFD_INTEROPERABILITY, d, ds, o, recursion_depth + 1);
 				break;
 			case EXIF_TAG_JPEG_INTERCHANGE_FORMAT:
 				thumbnail_offset = o;
 				if (thumbnail_offset && thumbnail_length)
 					exif_data_load_data_thumbnail (data, d,
-						ds, thumbnail_offset,
-						thumbnail_length);
+								       ds, thumbnail_offset,
+								       thumbnail_length);
 				break;
 			case EXIF_TAG_JPEG_INTERCHANGE_FORMAT_LENGTH:
 				thumbnail_length = o;
 				if (thumbnail_offset && thumbnail_length)
 					exif_data_load_data_thumbnail (data, d,
-						ds, thumbnail_offset,
-						thumbnail_length);
+								       ds, thumbnail_offset,
+								       thumbnail_length);
 				break;
 			default:
 				return;
@@ -324,20 +389,62 @@ exif_data_load_data_content (ExifData *data, ExifContent *ifd,
 		default:
 
 			/*
-			 * If we don't know the tag, changes are high
-			 * that the EXIF data does not follow the standard.
+			 * If we don't know the tag, don't fail. It could be that new 
+			 * versions of the standard have defined additional tags. Note that
+			 * 0 is a valid tag in the GPS IFD.
 			 */
-			if (!exif_tag_get_name (tag)) return;
+			if (!exif_tag_get_name_in_ifd (tag, ifd)) {
+
+				/*
+				 * Special case: Tag and format 0. That's against specification.
+				 * At least up to 2.2. But Photoshop writes it anyways.
+				 */
+				if (!memcmp (d + offset + 12 * i, "\0\0\0\0", 4)) {
+					exif_log (data->priv->log, EXIF_LOG_CODE_DEBUG, "ExifData",
+						  "Skipping empty entry at position %i in '%s'.", i, 
+						  exif_ifd_get_name (ifd));
+					break;
+				}
+				exif_log (data->priv->log, EXIF_LOG_CODE_DEBUG, "ExifData",
+					  "Unknown tag 0x%04x (entry %i in '%s'). Please report this tag "
+					  "to <libexif-devel@lists.sourceforge.net>.", tag, i,
+					  exif_ifd_get_name (ifd));
+				if (data->priv->options & EXIF_DATA_OPTION_IGNORE_UNKNOWN_TAGS)
+					break;
+			}
 			entry = exif_entry_new_mem (data->priv->mem);
-			exif_content_add_entry (ifd, entry);
-			exif_data_load_data_entry (data, entry, d, ds,
-						   offset + 12 * i);
+			if (exif_data_load_data_entry (data, entry, d, ds,
+						   offset + 12 * i))
+				exif_content_add_entry (data->ifd[ifd], entry);
 			exif_entry_unref (entry);
 			break;
 		}
 	}
 }
 
+static int
+cmp_func (const unsigned char *p1, const unsigned char *p2, ExifByteOrder o)
+{
+	ExifShort tag1 = exif_get_short (p1, o);
+	ExifShort tag2 = exif_get_short (p2, o);
+
+	return (tag1 < tag2) ? -1 : (tag1 > tag2) ? 1 : 0;
+}
+
+static int
+cmp_func_intel (const void *elem1, const void *elem2)
+{
+	return cmp_func ((const unsigned char *) elem1,
+			 (const unsigned char *) elem2, EXIF_BYTE_ORDER_INTEL);
+}
+
+static int
+cmp_func_motorola (const void *elem1, const void *elem2)
+{
+	return cmp_func ((const unsigned char *) elem1,
+			 (const unsigned char *) elem2, EXIF_BYTE_ORDER_MOTOROLA);
+}
+
 static void
 exif_data_save_data_content (ExifData *data, ExifContent *ifd,
 			     unsigned char **d, unsigned int *ds,
@@ -346,7 +453,8 @@ exif_data_save_data_content (ExifData *data, ExifContent *ifd,
 	unsigned int j, n_ptr = 0, n_thumb = 0;
 	ExifIfd i;
 
-	if (!data || !data->priv || !ifd || !d || !ds) return;
+	if (!data || !data->priv || !ifd || !d || !ds) 
+		return;
 
 	for (i = 0; i < EXIF_IFD_COUNT; i++)
 		if (ifd == data->ifd[i])
@@ -405,8 +513,8 @@ exif_data_save_data_content (ExifData *data, ExifContent *ifd,
 		  "Saving %i entries (IFD '%s', offset: %i)...",
 		  ifd->count, exif_ifd_get_name (i), offset);
 	for (j = 0; j < ifd->count; j++)
-		exif_data_save_data_entry (data, ifd->entries[j],
-				d, ds, offset + 12 * j);
+		exif_data_save_data_entry (data, ifd->entries[j], d, ds, offset + 12 * j);
+
 	offset += 12 * ifd->count;
 
 	/* Now save special entries. */
@@ -430,7 +538,7 @@ exif_data_save_data_content (ExifData *data, ExifContent *ifd,
 			exif_set_long  (*d + 6 + offset + 8, data->priv->order,
 					*ds - 6);
 			exif_data_save_data_content (data,
-				data->ifd[EXIF_IFD_EXIF], d, ds, *ds - 6);
+						     data->ifd[EXIF_IFD_EXIF], d, ds, *ds - 6);
 			offset += 12;
 		}
 
@@ -445,7 +553,7 @@ exif_data_save_data_content (ExifData *data, ExifContent *ifd,
 			exif_set_long  (*d + 6 + offset + 8, data->priv->order,
 					*ds - 6);
 			exif_data_save_data_content (data,
-				data->ifd[EXIF_IFD_GPS], d, ds, *ds - 6);
+						     data->ifd[EXIF_IFD_GPS], d, ds, *ds - 6);
 			offset += 12;
 		}
 
@@ -466,8 +574,8 @@ exif_data_save_data_content (ExifData *data, ExifContent *ifd,
 			exif_set_long  (*d + 6 + offset + 8, data->priv->order,
 					*ds - 6);
 			exif_data_save_data_content (data,
-				data->ifd[EXIF_IFD_INTEROPERABILITY], d, ds,
-				*ds - 6);
+						     data->ifd[EXIF_IFD_INTEROPERABILITY], d, ds,
+						     *ds - 6);
 			offset += 12;
 		}
 
@@ -501,7 +609,7 @@ exif_data_save_data_content (ExifData *data, ExifContent *ifd,
 
 			/* EXIF_TAG_JPEG_INTERCHANGE_FORMAT_LENGTH */
 			exif_set_short (*d + 6 + offset + 0, data->priv->order,
-				EXIF_TAG_JPEG_INTERCHANGE_FORMAT_LENGTH);
+					EXIF_TAG_JPEG_INTERCHANGE_FORMAT_LENGTH);
 			exif_set_short (*d + 6 + offset + 2, data->priv->order,
 					EXIF_FORMAT_LONG);
 			exif_set_long  (*d + 6 + offset + 4, data->priv->order,
@@ -516,9 +624,14 @@ exif_data_save_data_content (ExifData *data, ExifContent *ifd,
 		break;
 	}
 
+	/* Sort the directory according to TIFF specification */
+	qsort (*d + 6 + offset - (ifd->count + n_ptr + n_thumb) * 12,
+	       (ifd->count + n_ptr + n_thumb), 12,
+	       (data->priv->order == EXIF_BYTE_ORDER_INTEL) ? cmp_func_intel : cmp_func_motorola);
+
 	/* Correctly terminate the directory */
 	if (i == EXIF_IFD_0 && (data->ifd[EXIF_IFD_1]->count ||
-					     data->size)) {
+				data->size)) {
 
 		/*
 		 * We are saving IFD 0. Tell where IFD 1 starts and save
@@ -531,6 +644,56 @@ exif_data_save_data_content (ExifData *data, ExifContent *ifd,
 		exif_set_long (*d + 6 + offset, data->priv->order, 0);
 }
 
+typedef enum {
+	EXIF_DATA_TYPE_MAKER_NOTE_NONE		= 0,
+	EXIF_DATA_TYPE_MAKER_NOTE_CANON		= 1,
+	EXIF_DATA_TYPE_MAKER_NOTE_OLYMPUS	= 2,
+	EXIF_DATA_TYPE_MAKER_NOTE_PENTAX	= 3
+} ExifDataTypeMakerNote;
+
+static ExifDataTypeMakerNote
+exif_data_get_type_maker_note (ExifData *d)
+{
+	ExifEntry *e, *em;
+	char value[1024];
+
+	if (!d) 
+		return EXIF_DATA_TYPE_MAKER_NOTE_NONE;
+	
+	e = exif_data_get_entry (d, EXIF_TAG_MAKER_NOTE);
+        if (!e) 
+		return EXIF_DATA_TYPE_MAKER_NOTE_NONE;
+
+	/* Olympus & Nikon */
+	if ((e->size >= 8) && (!memcmp (e->data, "OLYMP", 6) ||
+			!memcmp (e->data, "OLYMPUS", 8) || !memcmp (e->data, "Nikon", 6)))
+		return EXIF_DATA_TYPE_MAKER_NOTE_OLYMPUS;
+
+	em = exif_data_get_entry (d, EXIF_TAG_MAKE);
+	if (!em) 
+		return EXIF_DATA_TYPE_MAKER_NOTE_NONE;
+
+	/* Canon */
+	if (!strcmp (exif_entry_get_value (em, value, sizeof (value)), "Canon"))
+		return EXIF_DATA_TYPE_MAKER_NOTE_CANON;
+
+	/* Pentax & some variant of Nikon */
+	if ((e->size >= 2) && (e->data[0] == 0x00) && (e->data[1] == 0x1b)) {
+		if (!strncasecmp (
+			    exif_entry_get_value (em, value, sizeof(value)),
+			    "Nikon", 5))
+			return EXIF_DATA_TYPE_MAKER_NOTE_OLYMPUS;
+		else
+			return EXIF_DATA_TYPE_MAKER_NOTE_PENTAX;
+	}
+
+	return EXIF_DATA_TYPE_MAKER_NOTE_NONE;
+}
+
+#define LOG_TOO_SMALL \
+exif_log (data->priv->log, EXIF_LOG_CODE_CORRUPT_DATA, "ExifData", \
+		_("Size of data too small to allow for EXIF data."));
+
 void
 exif_data_load_data (ExifData *data, const unsigned char *d_orig,
 		     unsigned int ds_orig)
@@ -538,11 +701,11 @@ exif_data_load_data (ExifData *data, const unsigned char *d_orig,
 	unsigned int l;
 	ExifLong offset;
 	ExifShort n;
-	ExifEntry *e, *em;
 	const unsigned char *d = d_orig;
 	unsigned int ds = ds_orig, len;
 
-	if (!data || !data->priv || !d || !ds) return;
+	if (!data || !data->priv || !d || !ds) 
+		return;
 
 	exif_log (data->priv->log, EXIF_LOG_CODE_DEBUG, "ExifData",
 		  "Parsing %i byte(s) EXIF data...\n", ds);
@@ -552,8 +715,7 @@ exif_data_load_data (ExifData *data, const unsigned char *d_orig,
 	 * not, search the EXIF marker.
 	 */
 	if (ds < 6) {
-		exif_log (data->priv->log, EXIF_LOG_CODE_DEBUG, "ExifData",
-		          "Size too small.");
+		LOG_TOO_SMALL;
 		return;
 	}
 	if (!memcmp (d, ExifHeader, 6)) {
@@ -590,15 +752,14 @@ exif_data_load_data (ExifData *data, const unsigned char *d_orig,
 				break;
 
 			/* Unknown marker or data. Give up. */
-			exif_log (data->priv->log, EXIF_LOG_CODE_DEBUG,
-				  "ExifData", "EXIF marker not found.");
+			exif_log (data->priv->log, EXIF_LOG_CODE_CORRUPT_DATA,
+				  "ExifData", _("EXIF marker not found."));
 			return;
 		}
 		d++;
 		ds--;
 		if (ds < 2) {
-			exif_log (data->priv->log, EXIF_LOG_CODE_DEBUG,
-				  "ExifData", "Size too small.");
+			LOG_TOO_SMALL;
 			return;
 		}
 		len = (d[0] << 8) | d[1];
@@ -614,13 +775,12 @@ exif_data_load_data (ExifData *data, const unsigned char *d_orig,
 	 * (offset 2, length 6).
 	 */
 	if (ds < 6) {
-		exif_log (data->priv->log, EXIF_LOG_CODE_DEBUG, "ExifData",
-			  "Size too small.");
+		LOG_TOO_SMALL;
 		return;
 	}
 	if (memcmp (d, ExifHeader, 6)) {
-		exif_log (data->priv->log, EXIF_LOG_CODE_DEBUG, "ExifData",
-			  "EXIF header not found.");
+		exif_log (data->priv->log, EXIF_LOG_CODE_CORRUPT_DATA,
+			  "ExifData", _("EXIF header not found."));
 		return;
 	}
 
@@ -628,14 +788,17 @@ exif_data_load_data (ExifData *data, const unsigned char *d_orig,
 		  "Found EXIF header.");
 
 	/* Byte order (offset 6, length 2) */
-	if (ds < 12)
+	if (ds < 14)
 		return;
 	if (!memcmp (d + 6, "II", 2))
 		data->priv->order = EXIF_BYTE_ORDER_INTEL;
 	else if (!memcmp (d + 6, "MM", 2))
 		data->priv->order = EXIF_BYTE_ORDER_MOTOROLA;
-	else
+	else {
+		exif_log (data->priv->log, EXIF_LOG_CODE_CORRUPT_DATA,
+			  "ExifData", _("Unknown encoding."));
 		return;
+	}
 
 	/* Fixed value */
 	if (exif_get_short (d + 8, data->priv->order) != 0x002a)
@@ -646,12 +809,17 @@ exif_data_load_data (ExifData *data, const unsigned char *d_orig,
 	exif_log (data->priv->log, EXIF_LOG_CODE_DEBUG, "ExifData", 
 		  "IFD 0 at %i.", (int) offset);
 
-	/* Parse the actual exif data (offset 14) */
-	exif_data_load_data_content (data, data->ifd[EXIF_IFD_0], d + 6,
-				     ds - 6, offset);
+	/* Parse the actual exif data (usually offset 14 from start) */
+	exif_data_load_data_content (data, EXIF_IFD_0, d + 6, ds - 6, offset, 0);
 
 	/* IFD 1 offset */
+	if (offset + 6 + 2 > ds) {
+		return;
+	}
 	n = exif_get_short (d + 6 + offset, data->priv->order);
+	if (offset + 6 + 2 + 12 * n + 4 > ds) {
+		return;
+	}
 	offset = exif_get_long (d + 6 + offset + 2 + 12 * n, data->priv->order);
 	if (offset) {
 		exif_log (data->priv->log, EXIF_LOG_CODE_DEBUG, "ExifData",
@@ -659,13 +827,12 @@ exif_data_load_data (ExifData *data, const unsigned char *d_orig,
 
 		/* Sanity check. */
 		if (offset > ds - 6) {
-			exif_log (data->priv->log, EXIF_LOG_CODE_DEBUG, 
-				  "ExifData", "Bogus offset!");
+			exif_log (data->priv->log, EXIF_LOG_CODE_CORRUPT_DATA,
+				  "ExifData", "Bogus offset.");
 			return;
 		}
 
-		exif_data_load_data_content (data, data->ifd[EXIF_IFD_1], d + 6,
-					     ds - 6, offset);
+		exif_data_load_data_content (data, EXIF_IFD_1, d + 6, ds - 6, offset, 0);
 	}
 
 	/*
@@ -674,47 +841,34 @@ exif_data_load_data (ExifData *data, const unsigned char *d_orig,
 	 * space between IFDs. Here is the only place where we have access
 	 * to that data.
 	 */
-	e = exif_data_get_entry (data, EXIF_TAG_MAKER_NOTE);
-	if (e) {
-
-	    /* Olympus & Nikon */
-	    if ((e->size >= 5) && (!memcmp (e->data, "OLYMP", 5) ||
-				   !memcmp (e->data, "Nikon", 5))) {
-			data->priv->md = exif_mnote_data_olympus_new (data->priv->mem);
-	    } else {
-			char value[7];
-			em = exif_data_get_entry (data, EXIF_TAG_MAKE);
-			/* Pentax & some variant of Nikon */
-			if ((e->size >= 2) && (e->data[0] == 0x00)
-				    && (e->data[1] == 0x1b)) {
-				if (em && !strncasecmp (exif_entry_get_value (em, value, sizeof(value)), "Nikon", 5)) {
-					data->priv->md = exif_mnote_data_olympus_new (data->priv->mem);
-				} else {
-					data->priv->md = exif_mnote_data_pentax_new (data->priv->mem);
-				}
-
-			} else {
-				/* Canon */
-				if (em) {
-		
-					if (!strcmp (exif_entry_get_value (em, value, sizeof(value)), "Canon"))
-						data->priv->md = exif_mnote_data_canon_new (data->priv->mem);
-				}
-			}
-		}
-
-	    /* 
-	     * If we are able to interpret the maker note, do so.
-	     */
-	    if (data->priv->md) {
-		    exif_mnote_data_log (data->priv->md, data->priv->log);
-		    exif_mnote_data_set_byte_order (data->priv->md,
-				    		    data->priv->order);
-		    exif_mnote_data_set_offset (data->priv->md,
-				    		data->priv->offset_mnote);
-		    exif_mnote_data_load (data->priv->md, d, ds);
-	    }
+	switch (exif_data_get_type_maker_note (data)) {
+	case EXIF_DATA_TYPE_MAKER_NOTE_OLYMPUS:
+		data->priv->md = exif_mnote_data_olympus_new (data->priv->mem);
+		break;
+	case EXIF_DATA_TYPE_MAKER_NOTE_PENTAX:
+		data->priv->md = exif_mnote_data_pentax_new (data->priv->mem);
+		break;
+	case EXIF_DATA_TYPE_MAKER_NOTE_CANON:
+		data->priv->md = exif_mnote_data_canon_new (data->priv->mem, data->priv->options);
+		break;
+	default:
+		break;
 	}
+
+	/* 
+	 * If we are able to interpret the maker note, do so.
+	 */
+	if (data->priv->md) {
+		exif_mnote_data_log (data->priv->md, data->priv->log);
+		exif_mnote_data_set_byte_order (data->priv->md,
+						data->priv->order);
+		exif_mnote_data_set_offset (data->priv->md,
+					    data->priv->offset_mnote);
+		exif_mnote_data_load (data->priv->md, d, ds);
+	}
+
+	if (data->priv->options & EXIF_DATA_OPTION_FOLLOW_SPECIFICATION)
+		exif_data_fix (data);
 }
 
 void
@@ -726,7 +880,8 @@ exif_data_save_data (ExifData *data, unsigned char **d, unsigned int *ds)
 	/* Header */
 	*ds = 14;
 	*d = exif_data_alloc (data, *ds);
-	if (!*d) return;
+	if (!*d) 
+		return;
 	memcpy (*d, ExifHeader, 6);
 
 	/* Order (offset 6) */
@@ -782,11 +937,11 @@ exif_data_ref (ExifData *data)
 void
 exif_data_unref (ExifData *data)
 {
-	if (!data)
+	if (!data) 
 		return;
 
 	data->priv->ref_count--;
-	if (!data->priv->ref_count)
+	if (!data->priv->ref_count) 
 		exif_data_free (data);
 }
 
@@ -794,8 +949,10 @@ void
 exif_data_free (ExifData *data)
 {
 	unsigned int i;
+	ExifMem *mem = (data && data->priv) ? data->priv->mem : NULL;
 
-	if (!data) return;
+	if (!data) 
+		return;
 
 	for (i = 0; i < EXIF_IFD_COUNT; i++) {
 		if (data->ifd[i]) {
@@ -804,11 +961,15 @@ exif_data_free (ExifData *data)
 		}
 	}
 
+	if (data->data) {
+		exif_mem_free (mem, data->data);
+		data->data = NULL;
+	}
+
 	if (data->priv) {
-		ExifMem *mem = data->priv->mem;
-		if (data->data) {
-			exif_mem_free (data->priv->mem, data->data);
-			data->data = NULL;
+		if (data->priv->log) {
+			exif_log_unref (data->priv->log);
+			data->priv->log = NULL;
 		}
 		if (data->priv->md) {
 			exif_mnote_data_unref (data->priv->md);
@@ -816,8 +977,9 @@ exif_data_free (ExifData *data)
 		}
 		exif_mem_free (mem, data->priv);
 		exif_mem_free (mem, data);
-		exif_mem_unref (mem);
 	}
+
+	exif_mem_unref (mem);
 }
 
 void
@@ -878,75 +1040,11 @@ static void
 entry_set_byte_order (ExifEntry *e, void *data)
 {
 	ByteOrderChangeData *d = data;
-	unsigned int i;
-	ExifShort s;
-	ExifLong l;
-	ExifSLong sl;
-	ExifRational r;
-	ExifSRational sr;
 
 	if (!e)
 		return;
 
-	switch (e->format) {
-	case EXIF_FORMAT_SHORT:
-		for (i = 0; i < e->components; i++) {
-			s = exif_get_short (e->data +
-				(i * exif_format_get_size (e->format)),
-				d->old);
-			exif_set_short (e->data +
-				(i * exif_format_get_size (e->format)),
-				d->new, s);
-		}
-		break;
-	case EXIF_FORMAT_LONG:
-		for (i = 0; i < e->components; i++) {
-			l = exif_get_long (e->data +
-				(i * exif_format_get_size (e->format)),
-				d->old);
-			exif_set_long (e->data +
-				(i * exif_format_get_size (e->format)),
-				d->new, l);
-		}
-		break;
-	case EXIF_FORMAT_RATIONAL:
-		for (i = 0; i < e->components; i++) {
-			r = exif_get_rational (e->data +
-				(i * exif_format_get_size (e->format)),
-				d->old);
-			exif_set_rational (e->data +
-				(i * exif_format_get_size (e->format)),
-				d->new, r);
-		}
-		break;
-	case EXIF_FORMAT_SLONG:
-		for (i = 0; i < e->components; i++) {
-			sl = exif_get_slong (e->data +
-				(i * exif_format_get_size (e->format)),
-				d->old);
-			exif_set_slong (e->data +
-				(i * exif_format_get_size (e->format)),
-				d->new, sl);
-		}
-		break;
-	case EXIF_FORMAT_SRATIONAL:
-		for (i = 0; i < e->components; i++) {
-			sr = exif_get_srational (e->data +
-				(i * exif_format_get_size (e->format)),
-				d->old);
-			exif_set_srational (e->data +
-				(i * exif_format_get_size (e->format)),
-				d->new, sr);
-		}
-		break;
-	case EXIF_FORMAT_UNDEFINED:
-	case EXIF_FORMAT_BYTE:
-	case EXIF_FORMAT_SBYTE:
-	case EXIF_FORMAT_ASCII:
-	default:
-		/* Nothing here. */
-		break;
-	}
+	exif_array_set_byte_order (e->format, e->data, e->components, d->old, d->new);
 }
 
 static void
@@ -974,10 +1072,16 @@ exif_data_set_byte_order (ExifData *data, ExifByteOrder order)
 void
 exif_data_log (ExifData *data, ExifLog *log)
 {
-	if (!data || !data->priv) return;
+	unsigned int i;
+
+	if (!data || !data->priv) 
+		return;
 	exif_log_unref (data->priv->log);
 	data->priv->log = log;
 	exif_log_ref (log);
+
+	for (i = 0; i < EXIF_IFD_COUNT; i++)
+		exif_content_log (data->ifd[i], log);
 }
 
 /* Used internally within libexif */
@@ -985,6 +1089,104 @@ ExifLog *exif_data_get_log (ExifData *);
 ExifLog *
 exif_data_get_log (ExifData *data)
 {
-	if (!data || !data->priv) return NULL;
+	if (!data || !data->priv) 
+		return NULL;
 	return data->priv->log;
 }
+
+static struct {
+	ExifDataOption option;
+	const char *name;
+	const char *description;
+} exif_data_option[] = {
+	{EXIF_DATA_OPTION_IGNORE_UNKNOWN_TAGS, N_("Ignore unknown tags"),
+	 N_("Ignore unknown tags when loading EXIF data.")},
+	{EXIF_DATA_OPTION_FOLLOW_SPECIFICATION, N_("Follow specification"),
+	 N_("Add, correct and remove entries to get EXIF data that follows "
+	    "the specification.")},
+	{EXIF_DATA_OPTION_DONT_CHANGE_MAKER_NOTE, N_("Don not change maker note"),
+	 N_("When loading and resaving Exif data, save the maker note unmodified."
+	    " Be aware that the maker note can get corrupted.")},
+	{0, NULL, NULL}
+};
+
+const char *
+exif_data_option_get_name (ExifDataOption o)
+{
+	unsigned int i;
+
+	for (i = 0; exif_data_option[i].name; i++)
+		if (exif_data_option[i].option == o) 
+			break;
+	return _(exif_data_option[i].name);
+}
+
+const char *
+exif_data_option_get_description (ExifDataOption o)
+{
+	unsigned int i;
+
+	for (i = 0; exif_data_option[i].description; i++)
+		if (exif_data_option[i].option == o) 
+			break;
+	return _(exif_data_option[i].description);
+}
+
+void
+exif_data_set_option (ExifData *d, ExifDataOption o)
+{
+	if (!d) 
+		return;
+
+	d->priv->options |= o;
+}
+
+void
+exif_data_unset_option (ExifData *d, ExifDataOption o)
+{
+	if (!d) 
+		return;
+
+	d->priv->options &= ~o;
+}
+
+static void
+fix_func (ExifContent *c, void *data)
+{
+	switch (exif_content_get_ifd (c)) {
+	case EXIF_IFD_1:
+		if (c->parent->data)
+			exif_content_fix (c);
+		else {
+			exif_log (c->parent->priv->log, EXIF_LOG_CODE_DEBUG, "exif-data",
+				  "No thumbnail but entries on thumbnail. These entries have been "
+				  "removed.");
+			while (c->count)
+				exif_content_remove_entry (c, c->entries[c->count - 1]);
+		}
+		break;
+	default:
+		exif_content_fix (c);
+	}
+}
+
+void
+exif_data_fix (ExifData *d)
+{
+	exif_data_foreach_content (d, fix_func, NULL);
+}
+
+void
+exif_data_set_data_type (ExifData *d, ExifDataType dt)
+{
+	if (!d || !d->priv) 
+		return;
+
+	d->priv->data_type = dt;
+}
+
+ExifDataType
+exif_data_get_data_type (ExifData *d)
+{
+	return (d && d->priv) ? d->priv->data_type : EXIF_DATA_TYPE_COUNT;
+}
