@@ -395,7 +395,9 @@ static struct {
       N_("right - bottom"), N_("left - bottom"), NULL}},
   { EXIF_TAG_YCBCR_POSITIONING,
     { "", N_("centered"), N_("co-sited"), NULL}},
-  { EXIF_TAG_PHOTOMETRIC_INTERPRETATION, {"", N_("RGB"), N_("YCbCr"), NULL}},
+  { EXIF_TAG_PHOTOMETRIC_INTERPRETATION,
+    {N_("Reversed mono"), N_("Normal mono"), N_("RGB"), N_("Palette"), "",
+      N_("CMYK"), N_("YCbCr"), "", N_("CieLAB"), NULL}},
   { EXIF_TAG_CUSTOM_RENDERED,
     { N_("Normal process"), N_("Custom process"), NULL}},
   { EXIF_TAG_EXPOSURE_MODE,
@@ -436,6 +438,9 @@ static struct {
     { {1, {N_("Uncompressed"), NULL}},
       {5, {N_("LZW compression"), NULL}},
       {6, {N_("JPEG compression"), NULL}},
+      {7, {N_("JPEG compression"), NULL}},
+      {8, {N_("Deflate/ZIP compression"), NULL}},
+      {32773, {N_("PackBits compression"), NULL}},
       {0, {NULL}}}},
   { EXIF_TAG_LIGHT_SOURCE,
     { {  0, {N_("Unknown"), NULL}},
@@ -488,12 +493,13 @@ static struct {
       {0x0005, {N_("Strobe return light not detected."), N_("W/o strobe"),
 		NULL}},
       {0x0007, {N_("Strobe return light detected."), N_("W. strobe"), NULL}},
-      {0x0009, {N_("Flash fired, compulsatory flash mode"), NULL}},
-      {0x000d, {N_("Flash fired, compulsatory flash mode, return light "
+      {0x0008, {N_("Flash did not fire."), NULL}}, /* Olympus E-330 */
+      {0x0009, {N_("Flash fired, compulsory flash mode."), NULL}},
+      {0x000d, {N_("Flash fired, compulsory flash mode, return light "
 		   "not detected."), NULL}},
-      {0x000f, {N_("Flash fired, compulsatory flash mode, return light "
+      {0x000f, {N_("Flash fired, compulsory flash mode, return light "
 		   "detected."), NULL}},
-      {0x0010, {N_("Flash did not fire, compulsatory flash mode."), NULL}},
+      {0x0010, {N_("Flash did not fire, compulsory flash mode."), NULL}},
       {0x0018, {N_("Flash did not fire, auto mode."), NULL}},
       {0x0019, {N_("Flash fired, auto mode."), NULL}},
       {0x001d, {N_("Flash fired, auto mode, return light not detected."),
@@ -508,11 +514,11 @@ static struct {
       {0x0049, {N_("Flash fired, compulsory flash mode, red-eye reduction "
 		   "mode."), NULL}},
       {0x004d, {N_("Flash fired, compulsory flash mode, red-eye reduction "
-		  "mode, return light not detected"), NULL}},
-      {0x004f, {N_("Flash fired, compulsory flash mode, red-eye reduction, "
-		   "return light detected"), NULL}},
-      {0x0058, {N_("Flash did not fire, auto mode, red-eye reduction mode"), NULL}},
-      {0x0059, {N_("Flash fired, auto mode, red-eye reduction mode"), NULL}},
+		  "mode, return light not detected."), NULL}},
+      {0x004f, {N_("Flash fired, compulsory flash mode, red-eye reduction mode, "
+		   "return light detected."), NULL}},
+      {0x0058, {N_("Flash did not fire, auto mode, red-eye reduction mode."), NULL}},
+      {0x0059, {N_("Flash fired, auto mode, red-eye reduction mode."), NULL}},
       {0x005d, {N_("Flash fired, auto mode, return light not detected, "
 		   "red-eye reduction mode."), NULL}},
       {0x005f, {N_("Flash fired, auto mode, return light detected, "
@@ -527,7 +533,8 @@ static struct {
   { EXIF_TAG_COLOR_SPACE,
     { {1, {N_("sRGB"), NULL}},
       {2, {N_("Adobe RGB"), NULL}},
-      {0xffff, {N_("Uncalibrated"), NULL}}}},
+      {0xffff, {N_("Uncalibrated"), NULL}},
+      {0x0000, {NULL}}}},
   {0, }
 };
 
@@ -704,13 +711,16 @@ exif_entry_get_value (ExifEntry *e, char *val, unsigned int maxlen)
 						    (float) v_rat.denominator);
 		break;
 	case EXIF_TAG_APERTURE_VALUE:
+  case EXIF_TAG_MAX_APERTURE_VALUE:
 		CF (e, EXIF_FORMAT_RATIONAL, val, maxlen);
 		CC (e, 1, val, maxlen);
 		v_rat = exif_get_rational (e->data, o);
 		if (!v_rat.denominator) return val;
-		snprintf (val, maxlen, "f/%.01f",
-			  pow (2 , ((float) v_rat.numerator /
-				    (float) v_rat.denominator) / 2.));
+    d = (double) v_rat.numerator / (double) v_rat.denominator;
+    snprintf (val, maxlen, _("%.02f EV"), d);
+    snprintf (b, sizeof (b), _(" (f/%.01f)"), pow (2, d / 2.));
+    if (maxlen > strlen (val) + strlen (b))
+      strncat (val, b, maxlen - strlen (val) - 1);
 		break;
 	case EXIF_TAG_FOCAL_LENGTH:
 		CF (e, EXIF_FORMAT_RATIONAL, val, maxlen);
@@ -775,23 +785,32 @@ exif_entry_get_value (ExifEntry *e, char *val, unsigned int maxlen)
 		CC (e, 1, val, maxlen);
 		v_srat = exif_get_srational (e->data, o);
 		if (!v_srat.denominator) return val;
-		snprintf (val, maxlen, "%.0f/%.0f", (float) v_srat.numerator,
-			  (float) v_srat.denominator);
-		if (maxlen > strlen (val) + strlen (_(" sec.")))
-			strncat (val, _(" sec."), maxlen - strlen (val) - 1);
-		snprintf (b, sizeof (b), " (APEX: %i)",
-			(int) pow (sqrt(2), (float) v_srat.numerator /
-				            (float) v_srat.denominator));
+    d = (double) v_srat.numerator / (double) v_srat.denominator;
+    snprintf (val, maxlen, _("%.02f EV"), d);
+		snprintf (b, sizeof (b), " (APEX: %i)", (int) pow (sqrt(2), d));
 		if (maxlen > strlen (val) + strlen (b))
 			strncat (val, b, maxlen - strlen (val) - 1);
+    d = 1. / pow (2, d);
+    if (d < 1)
+      snprintf (b, sizeof (b), _(" 1/%d sec.)"), (int) (1. / d));
+    else
+      snprintf (b, sizeof (b), _(" %d sec.)"), (int) d);
+    if (maxlen > strlen (val) + strlen (b)) {
+      val[strlen (val) - 1] = ',';
+      strncat (val, b, maxlen - strlen (val) - 1);
+    }
 		break;
 	case EXIF_TAG_BRIGHTNESS_VALUE:
 		CF (e, EXIF_FORMAT_SRATIONAL, val, maxlen);
 		CC (e, 1, val, maxlen);
 		v_srat = exif_get_srational (e->data, o);
-		snprintf (val, maxlen, "%i/%i", (int) v_srat.numerator,
-						  (int) v_srat.denominator);
-		/* FIXME: How do I calculate the APEX value? */
+    if (!v_srat.denominator) return val;
+    d = (double) v_srat.numerator / (double) v_srat.denominator;
+    snprintf (val, maxlen, _("%.02f EV"), d);
+    snprintf (b, sizeof (b), _(" (%.02f cd/m^2)"),
+        1. / (M_PI * 0.3048 * 0.3048) * pow (2, d));
+    if (maxlen > strlen (val) + strlen (b))
+      strncat (val, b, maxlen - strlen (val) - 1);
 		break;
 	case EXIF_TAG_FILE_SOURCE:
 		CF (e, EXIF_FORMAT_UNDEFINED, val, maxlen);
@@ -824,10 +843,8 @@ exif_entry_get_value (ExifEntry *e, char *val, unsigned int maxlen)
 		CC (e, 1, val, maxlen);
 		v_srat = exif_get_srational (e->data, o);
 		if (!v_srat.denominator) return val;
-		snprintf (val, maxlen, "%s%.01f",
-			  v_srat.denominator * v_srat.numerator > 0 ? "+" : "",
-			  (double) v_srat.numerator /
-			  (double) v_srat.denominator);
+    d = (double) v_srat.numerator / (double) v_srat.denominator;
+    snprintf (val, maxlen, _("%.02f EV"), d);
 		break;
 	case EXIF_TAG_YCBCR_SUB_SAMPLING:
 		CF (e, EXIF_FORMAT_SHORT, val, maxlen);
@@ -902,9 +919,9 @@ exif_entry_get_value (ExifEntry *e, char *val, unsigned int maxlen)
 		}
 
 		/* Find the value */
-		for (j = 0; list2[i].elem[j].values &&
+		for (j = 0; list2[i].elem[j].values[0] &&
 			    (list2[i].elem[j].index < v_short); j++);
-		if (list2[i].elem[j].index != v_short || v_short == 65535) {
+		if (list2[i].elem[j].index != v_short) {
 			snprintf (val, maxlen, _("Internal error (unknown "
 				  "value %i)."), v_short);
 			break;
@@ -912,8 +929,7 @@ exif_entry_get_value (ExifEntry *e, char *val, unsigned int maxlen)
 
 		/* Find a short enough value */
 		memset (val, 0, maxlen);
-		for (k = 0; list2[i].elem[j].values &&
-			    list2[i].elem[j].values[k]; k++) {
+		for (k = 0; list2[i].elem[j].values[k]; k++) {
 		  l = strlen (_(list2[i].elem[j].values[k]));
 		  if ((maxlen > l) && (strlen (val) < l))
 		    strncpy (val, _(list2[i].elem[j].values[k]), maxlen - 1);
@@ -952,6 +968,10 @@ exif_entry_get_value (ExifEntry *e, char *val, unsigned int maxlen)
 		else
 			strncpy (val, _(list[i].strings[j]), maxlen - 1);
 		break; 	
+	case EXIF_TAG_INTEROPERABILITY_VERSION:
+		CF (e, EXIF_FORMAT_UNDEFINED, val, maxlen);
+		strncpy (val, (char *) e->data, MIN (maxlen, e->size));
+		break;
 	default:
 		if (!e->components) break;
 		switch (e->format) {
